
# q-expansion of newform 5408.2.a.be, downloaded from the LMFDB on 14 January 2025.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 5408
weight = 2
poly_data = [-2, 0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0, 0], [1, 1], [0, 2], [-3, -1], [1, -3], [0, 0], [-3, -2], [-3, 3], [3, -3], [3, 4], [0, -4], [1, -6], [-3, -2], [3, -5], [-6, 0], [0, -2], [-3, 3], [7, 0], [-9, -3], [3, 3], [4, 6], [6, 0], [-4, 0], [-9, 4], [-9, 0], [9, 2], [0, 0], [-15, -3], [0, -6], [-3, 2], [-3, 1], [4, 0], [15, -4], [-3, 3], [-9, 4], [0, 12], [8, -6], [-21, 3], [3, 9], [-9, 0], [17, -3], [8, -6], [-11, 3], [-11, -6], [3, 0], [3, 15], [-15, 1], [3, -7], [-3, -9], [0, 6], [-12, -6], [-8, 12], [5, 18], [-5, -9], [-3, 14], [-5, -3], [-3, 6], [-21, -3], [5, 6], [12, -10], [9, 1], [-9, 8], [-12, 0], [-24, 0], [12, 6], [-6, -8], [3, 3], [-4, 6], [27, 3], [9, 18], [-9, -4], [8, 0], [-21, 9], [29, 6], [9, 13], [-3, -9], [6, -16], [-3, -6], [-3, 10], [-19, 6], [-1, 9], [-8, 6], [5, 21], [-11, 6], [21, 5], [12, 12], [21, -6], [31, 0], [21, -14], [-24, 4], [10, -12], [13, -3], [3, 7], [-1, -15], [12, 4], [-19, 3], [9, -10], [-12, 2], [15, 1], [-2, 0], [-36, 0], [3, 4], [-3, -21], [-9, -12], [-12, -20], [-4, 6], [-9, 21], [12, 2], [-14, -24], [-19, 6], [3, -1], [-23, 6], [-33, -8], [-12, 12], [3, 3], [-3, -14], [-21, 15], [-19, -3], [9, 10], [3, 15], [1, -18], [-25, 12], [0, -18], [5, 3], [9, 5], [0, -6], [-11, -6], [-27, 3], [24, 8], [10, 24], [-9, -15], [-3, -3], [27, -7], [-15, -6], [21, 6], [-3, -30], [-3, -6], [3, -29], [-33, 0], [-3, -2], [18, -4], [3, 12], [-21, 15], [12, -24], [19, 12], [19, -21], [-16, -18], [36, -2], [-36, 0], [32, 0], [7, 12], [21, -2], [-12, 28], [-3, -15], [-15, -15], [-24, 0], [-3, -25], [21, -22], [-20, -18], [-18, 0], [1, -3], [-9, 4], [-24, 12], [-37, 3], [15, 20], [16, -12], [-3, -39], [-1, -24], [-12, 18], [-24, 14], [-12, 12], [31, -12], [13, 27], [-11, 18], [0, -20], [-3, 2], [-9, 21], [-24, -2], [21, 21], [17, -18], [6, 0], [12, 12], [-16, 6], [36, 14], [43, 9], [21, 2], [-16, 18], [-12, -8], [47, 0], [11, -21], [-11, 30], [27, 15], [-6, 20], [-45, 12], [-53, 3], [-3, 30], [2, 0], [-17, -12], [-12, -10], [40, 12], [-3, -10], [-3, 31], [-13, 12], [28, -18], [15, 9], [3, -4], [-6, -32], [27, 27], [-27, -2], [-27, -9], [7, -36], [18, -8], [21, -27], [-3, -9], [29, 27], [-4, -6], [-24, -20], [-57, 4], [21, 9], [48, 6], [-37, 24], [24, 8], [-42, -16], [-21, -33], [7, 45], [16, -30], [-33, -8], [-3, 3], [-27, -17], [-18, 12], [-7, 6], [-33, 21], [-45, 1], [30, -16], [-36, 4], [-32, -24], [15, 36], [-21, -4], [53, 3], [-11, 33], [-9, 21], [-9, -3], [27, -5], [-13, 24], [-9, -20], [0, -24], [-27, -5], [39, -3], [21, -1], [-21, 9], [-15, 18], [39, 0], [-24, 36], [-25, 0], [0, 42], [37, 15], [-57, 0], [-33, -11], [24, 30], [7, 24], [24, 0], [39, 9], [8, -6], [-29, -24], [39, -4], [51, -21], [-33, -24], [-12, -38], [-3, -21], [21, -22], [-32, 6], [12, -12], [13, 6], [3, -17], [-9, -36], [3, 57], [-27, 15], [-16, -6], [23, -12], [27, 27], [-11, 33], [-75, -7], [18, 24], [-7, -30], [12, -4], [0, 12], [20, 18], [48, -14], [-27, -13], [-27, -10], [-63, -6], [-37, 15], [-33, -28], [37, -21], [-51, -18], [0, -34], [24, -4], [-21, -44], [33, 21], [-39, 9], [37, 30], [42, -8], [3, 37], [-12, 12], [-51, 15], [47, 12], [-46, 12], [-8, 6], [3, -27], [56, -18], [5, 15], [33, -30], [-36, -38], [-15, 5], [-75, 3], [23, -24], [-21, 3], [-24, 24], [-35, -18], [21, -34], [-12, -24], [-36, 6], [63, 16], [-3, 41], [-18, -16], [31, 24], [-36, -12], [-21, -41], [-37, 33], [-15, 50], [-5, 12], [-24, 22], [-69, -15], [21, 27], [-57, 5], [-36, 12], [7, -12], [21, -30], [29, 6], [24, 36], [8, 18], [-57, 16], [6, -32], [51, 5], [-9, 0], [28, 48], [-40, -6], [-69, 3], [-51, 21], [45, -22], [54, 20], [-3, -54], [9, -38], [51, 27], [-13, -48], [12, 34], [35, -9], [21, 15], [36, 38], [-24, 12], [47, -12], [-33, 44], [-13, -27], [-15, -39], [15, 9], [15, -12], [-81, -12], [-3, 63], [-4, 18], [-35, -9], [51, 3], [0, 24], [-24, -2], [-69, 21], [29, 42], [-6, -12], [13, 3], [21, -42], [-33, -8], [-19, -42], [24, 6], [21, -2], [24, 4], [-36, 30], [-27, -9], [3, 33], [51, -25], [-72, -18], [-30, -4], [67, 15], [-49, -24], [27, 8], [20, 12], [-9, -43], [3, 15], [15, 24], [-27, 5], [-6, -48], [36, 12], [-45, -4], [-33, 12], [21, 6], [21, 9], [-84, -10], [15, -52], [-21, -33], [57, 6], [21, 14], [-84, -8], [9, -3], [4, -54], [-39, -38], [-19, -33], [-3, -29], [5, -30], [-18, -32], [-5, 27], [24, 36], [-75, 6], [21, 27], [27, -40], [16, 42], [-19, -51], [12, 36], [39, 12], [-9, 4], [60, 12], [60, 2], [-21, 39], [-35, -33]]
